package strategy.impl;


import game.Gamer;
import game.Poker;
import org.springframework.util.LinkedMultiValueMap;
import pokerenum.PokerLevelEnum;
import pokerenum.PokerValueEnum;
import result.StrategyResult;

import java.util.List;
import java.util.Set;

/**
 *
 */
public class RepeatStrategy extends Strategy {
    private static RepeatStrategy strategy = null;

    private RepeatStrategy() {

    }

    public static RepeatStrategy getStrategy() {
        if (strategy == null) {
            strategy = new RepeatStrategy();
        }
        return strategy;
    }

    @Override
    public StrategyResult mactch(Gamer gamer) {
        List<Poker> pokers = gamer.getPokers();
        LinkedMultiValueMap<Character, Poker> pokerValueMap = new LinkedMultiValueMap();
        for (Poker poker : pokers) {
            pokerValueMap.add(poker.getValueEnum().getOrderValue(), poker);
        }
        int size = pokerValueMap.size();
        Set keys = pokerValueMap.keySet();
        StrategyResult result = new StrategyResult();
        result.setPokerLevelEnum(PokerLevelEnum.DEFAULT);
        if (size == 2) {
            for (Object key : keys) {
                List<Poker> subPokers = pokerValueMap.get(key);
                result.setMaxUsefulPoker(subPokers.get(0));
                if (subPokers.size() == 2
                        || subPokers.size() == 3) {
                    result.setPokerLevelEnum(PokerLevelEnum.FULLHOUSE);
                    break;
                } else if (subPokers.size() == 1 ||
                        subPokers.size() == 4) {
                    result.setPokerLevelEnum(PokerLevelEnum.FOURKIND);
                    break;
                }
            }
        } else if (size == 3) {
            for (Object key : keys) {
                List<Poker> subPokers = pokerValueMap.get(key);
                result.setMaxUsefulPoker(subPokers.get(0));
                int subSize = subPokers.size();
                if (subSize == 3) {
                    System.out.println("三条");
                    result.setPokerLevelEnum(PokerLevelEnum.THREEKIND);
                    break;
                } else if (subSize == 2) {
                    System.out.println("两对");
                    result.setPokerLevelEnum(PokerLevelEnum.TWOPAIRS);
                    break;
                }
            }
        } else if (size == 4) {
            System.out.println("一对");
            result.setPokerLevelEnum(PokerLevelEnum.ONEPAIR);
            for (Object key : keys) {
                List<Poker> subPokers = pokerValueMap.get(key);
                if (subPokers.size() == 2) {
                    result.setMaxUsefulPoker(subPokers.get(0));
                    break;
                }
            }
        } else {
            result.setPokerLevelEnum(PokerLevelEnum.HIGHCARD);
            System.out.println("高牌");
        }
        return result;
    }
}
