package strategy.impl;

import com.google.common.collect.Range;
import com.google.common.collect.RangeSet;
import com.google.common.collect.TreeRangeSet;
import game.Gamer;
import game.Poker;
import pokerenum.PokerLevelEnum;
import pokerenum.PokerValueEnum;
import pokerenum.ResultEnum;
import result.StrategyResult;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * 顺子
 */
public class StraightStrategy extends Strategy {

    private static StraightStrategy strategy = null;

    private StraightStrategy() {

    }

    public static StraightStrategy getStrategy() {
        if (strategy == null) {
            strategy = new StraightStrategy();
        }
        return strategy;
    }

    @Override
    public StrategyResult mactch(Gamer gamer) {
        List<Poker> pokers = gamer.getPokers();
        TreeRangeSet<Integer> pokerValue = TreeRangeSet.create();
        Poker maxPoker = null;
        for (Poker poker : pokers) {
            Range range = Range.open(poker.getValueEnum().getValue()-1, poker.getValueEnum().getValue()+1);
            pokerValue.add(range);
            if (maxPoker == null) {
                maxPoker = poker;
            } else if (maxPoker.getValueEnum().getValue() < poker.getValueEnum().getValue()) {
                maxPoker = poker;
            }
        }
        Set<Range<Integer>> ranges = pokerValue.asRanges();
        StrategyResult result = new StrategyResult();
        result.setMatched(false);
        result.setPokerLevelEnum(PokerLevelEnum.DEFAULT);
        if (ranges.size() == 1) {
            result.setMatched(true);
            result.setPokerLevelEnum(PokerLevelEnum.STRAIGHT);
            result.setMaxUsefulPoker(maxPoker);
            return result;
        }
        return result;
    }
}
